#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e10|e14|e15> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
        
    PRODUCTS=${product_dir}:${PRODUCTS}

    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi
if [ -z ${basequal} ]
then
   echo "ERROR: please specify the qualifiers"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=igprof
origpkgver=v5_9_16
pkgver=${origpkgver}b
ssibuildshims_version=v1_02_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
atomicdotver=7_4_0
winddotver=1.2
pkgtarname=igprof-${pkgdotver}
atomicname=libatomic_ops-libatomic_ops-${atomicdotver}
unwindname=libunwind-${winddotver}

get_this_dir

get_ssibuildshims

source define_basics NO_TYPE

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${basequal}
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${basequal} in
e9)  ;;
e1[045])  ;;
*) 
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
esac

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${basequal} || ssi_die "ERROR: fake declare failed"

setup ${package} ${pkgver} -q ${basequal} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed."
# We need cmake
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake "${product_dir}" || ssi_die "failed to setup cmake"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

export CC=gcc
export CXX=g++
cd ${blddir}
tar xf ${tardir}/${pkgtarname}.tar || ssi_die "Failed to unwind ${pkgtarname}.tar"
tar xf ${tardir}/${atomicname}.tar.bz2 || ssi_die "Failed to unwind ${atomicname}.tar.bz2"
tar xf ${tardir}/${unwindname}.tar.gz || ssi_die "Failed to unwind ${unwindname}.tar.gz"

# patch
patch -b -p0 < ${patchdir}/libunwind.patch || ssi_die "Failed to apply libunwind patch"

# build order: libatomic_ops, libunwind, igprof

set -x

cd ${blddir}/${atomicname} || ssi_die "ERROR: ${blddir}/${atomicname} does not exist"
./configure --prefix=${IGPROF_FQ_DIR} || ssi_die "ERROR: configure of libatomic_ops failed."
make -j ${ncore} install || ssi_die "ERROR: make of libatomic_ops failed."

cd ${blddir}/${unwindname} || ssi_die "ERROR: ${blddir}/${unwindname} does not exist"
./configure CFLAGS="-g -O3" CPPFLAGS="-I${pkgdir}/include" --prefix=${IGPROF_FQ_DIR} --disable-block-signals || ssi_die "ERROR: configure of libunwind failed."
make -j ${ncore}  || ssi_die "ERROR: make of libunwind failed."
make install || ssi_die "ERROR: install of libunwind failed."

cd ${blddir}/${pkgtarname} || ssi_die "ERROR: ${blddir}/${pkgtarname} does not exist"
cmake -DCMAKE_INSTALL_PREFIX=${IGPROF_FQ_DIR} \
      -DCMAKE_BUILD_TYPE=RelWithDebInfo \
      -DCMAKE_CXX_FLAGS="-g -O3 -std=c++14 -I${pkgdir}/include" \
      ${blddir}/${pkgtarname} || ssi_die "ERROR: cmake failed"
make -j ${ncore} install  || ssi_die "ERROR: build of igprof failed."

set +x


if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${basequal} || ssi_die "ERROR: declare failed"

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${basequal} -z ${product_dir}:${PRODUCTS}
echo "${package} is installed at ${IGPROF_DIR}"

# fixing pkgconfig file
for pkgcfg in libunwind-coredump.pc libunwind-generic.pc libunwind.pc libunwind-ptrace.pc libunwind-setjmp.pc
do
  mv ${IGPROF_FQ_DIR}/lib/pkgconfig/${pkgcfg} ${IGPROF_FQ_DIR}/lib/pkgconfig/${pkgcfg}.bak
  cat ${IGPROF_FQ_DIR}/lib/pkgconfig/${pkgcfg}.bak | sed "s%${IGPROF_FQ_DIR}%\${IGPROF_FQ_DIR}%g" > ${IGPROF_FQ_DIR}/lib/pkgconfig/${pkgcfg}
done

# remove libtool files since they contain hardwired paths
rm -vf ${IGPROF_FQ_DIR}/lib/*.la

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver}  ${basequal}
fi

exit 0
